{% extends 'userena/base_userena.html' %}
{% load i18n %}


{% block title %}{% trans "Account re-activation succeeded." %}{% endblock %}
{% block content_title %}<h2>{% trans "Account re-activation" %}</h2>{% endblock %}

{% block content %}
<p>{% blocktrans %}You requested a new activation of your account..{% endblocktrans %}</p>
<p>{% blocktrans %}You have been sent an e-mail with an activation link to the supplied email.{% endblocktrans %}</p>
<p>{% blocktrans %}We will store your signup information for {{ userena_activation_days }} days on our server. {% endblocktrans %}</p>
{% endblock %}
