{% extends 'userena/base_userena.html' %}
{% load i18n %}

{% block title %}{% trans "Account not activated" %}{% endblock %}
{% block content_title %}<h2>{% trans "Account not activated" %}</h2>{% endblock %}

{% block content %}
<p>{% trans "You have not confirmed your email address yet, please check your email for the activation link." %}</p>
<p>{% trans "If you are unable to complete the activation steps, please contact the administrators for assistance." %}</p>
{% endblock %}
