{% load i18n %}{% autoescape off %}
<html>
<body>
    {% if not without_usernames %}<p>{% blocktrans with user.username as username %}Dear {{ username }},</p>{% endblocktrans %}{% endif %}
    {% blocktrans with site.name as site %}<p>Thank you for signing up at {{ site }}.</p>{% endblocktrans %}
    <p>
        {% trans "To activate your account you should click on the link below:" %}<br />
        {{ protocol }}://{{ site.domain }}{% url 'userena_activate' activation_key %}
    </p>
    <p>
        {% trans "Thanks for using our site!" %}<br />
        {% trans "Sincerely" %},<br />
        {{ site.name }}
    </p>
</body>
</html>
{% endautoescape %}
