{% load i18n %}{% autoescape off %}
<html>
<body>
    {% if not without_usernames %}<p>{% blocktrans with user.username as username %}Dear {{ username }},</p>{% endblocktrans %}{% endif %}
    <p>
        {% blocktrans with site.name as site %}You requested a change of your email address at {{ site }}.{% endblocktrans %}
    </p>
    <p>
        {% trans "Please confirm this email address by clicking on the link below:" %}<br>
        {{ protocol }}://{{ site.domain }}{% url 'userena_email_confirm' confirmation_key %}
    </p>
    <p>
        {% trans "Thanks for using our site!" %}<br />
        {% trans "Sincerely" %},<br />
        {{ site_name }}
    </p>
    </body>
    </html>
{% endautoescape %}
