{% extends 'userena/base_userena.html' %}
{% load i18n %}

{% block title %}{% trans "Signup" %}{% endblock %}

{% block content %}
<form action="" method="post">
  {% csrf_token %}
  <fieldset>
    <legend>{% trans "Signup" %}</legend>
    {{ form.non_field_errors }}
    {% for field in form %}
    {{ field.errors }}
    {# Displaying checkboxes differently #}
    {% if field.name == 'tos' %}
    <p class="checkbox">
    <label for="id_{{ field.name }}">{{ field }} {{ field.label }}</label>
    </p>
    {% else %}
    <p>
    {{ field.label_tag }} 
    {{ field }}
    </p>
    {% endif %}
    {% endfor %}
  </fieldset>
  <input type="submit" value="{% trans "Signup"%}" />
</form>
{% endblock %}
