{% extends 'userena/base_userena.html' %}
{% load i18n %}

{% block title %}{% trans "Disabled account" %}{% endblock %}
{% block content_title %}<h2>{% trans "Your account has been disabled" %}</h2>{% endblock %}
{% block content %}
<p>{% trans "Your account has been disabled." %}</p>
<p>{% trans "Please contact the administrators if you require further assistance." %}</p>
{% endblock %} 
