{% load i18n %}{% autoescape off %}
<html>
<body>
    <p>
        {% blocktrans %}You're receiving this e-mail because you requested a password reset
            for your user account at {{ site_name }}{% endblocktrans %}.
    </p>
<p>
    {% trans "Please go to the following page and choose a new password:" %}<br />
    {% block reset_link %}
        <a rel="noreferrer" href="{{ protocol }}://{{ domain }}{% url 'auth_password_reset_confirm' uid token %}">
        {{ protocol }}://{{ domain }}{% url 'userena_password_reset_confirm' uid token %}
        </a>
    {% endblock %}
</p>
{% if not without_usernames %}
    <p>{% blocktrans with user.username as username %}Your username, in case you've forgotten: {{ username }}{% endblocktrans %}</p>
{% endif %}
<p>
    {% trans "Thanks for using our site!" %}<br />
    {% trans "Sincerely" %},<br />
    {{ site_name }}
</p>
</body>
</html>
{% endautoescape %}
