<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN"
"http://www.w3.org/TR/html4/strict.dtd">

<html lang="en">
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
  <title>{% block title %}Userena{% endblock %}</title>
  {% block meta %}{% endblock %}
</head>
<body id="{% block body_id %}{% endblock %}" class="{% block body_class %}{% endblock %}">
  <div id="body">
    {% block body %}
      <div class="content_title">
        {% block content_title %}{% endblock %}
      </div>
      <div class="content">
        {% block content %}{% endblock %}
      </div>
    {% endblock %}
  </div>
</body>
</html>
