{% extends 'userena/base_userena.html' %}
{% load i18n %}


{% block title %}{% trans "Account activation failed." %}{% endblock %}
{% block content_title %}<h2>{% trans "Your account could not be activated..." %}</h2>{% endblock %}

{% block content %}
<p>{% trans "Your account could not be activated as your activation link has expired." %}</p>
<p><a href="{% url 'userena_activate_retry' activation_key=activation_key %}">{% trans "Request a new activation link." %}</a></p>
{% endblock %}
