{% extends 'userena/base_userena.html' %}
{% load i18n %}

{% block title %}{% trans "Password reset" %}{% endblock %}
{% block content_title %}<h2>{% trans "Your password has been reset" %}</h2>{% endblock %}
{% block content %}
{% url 'userena_signin' as signin_url %}
{% blocktrans %}You can now <a href="{{ signin_url }}">signin</a> with your new password.{% endblocktrans %}
{% endblock %}
